# coding=utf-8

"""Checkers for CXX source files"""

import re

import aslint.common_checkers as COMM
from aslint.base_checkers import (FilenameCat, GenericMsg, TextMsg,
                                  checkers_from_context)


class MissingConstructor(FilenameCat, GenericMsg):

    """Missing constructor
    Each class should define a constructor used by users and a constructor
    required at restart that accepts the Jeveux name as argument.
    This checker expects at least two constructors per class. It may be
    unrelevant in some cases.
    """
    id = "C3006"
    fmt = "%(id)s: %(label)s"

    def search(self, filename):
        """Check that at least two constructors exist for each class."""
        error = []
        if not filename.endswith("Interface.cxx"):
            return error

        with open(filename, "r") as fobj:
            txt = fobj.read()

        expr1 = re.compile('^ *class_', re.M)
        expr2 = re.compile('initFactoryPtr', re.M)
        nbclass = len(expr1.findall(txt))
        nbinit = len(expr2.findall(txt))
        if nbinit < nbclass * 2:
            error.append(": {0} ({1} constructors missing)"
                         .format(filename, nbclass * 2 - nbinit))
        return error


CHECK_LIST = checkers_from_context(globals(), TextMsg)
